function CertificateAdjuster() {
}

CertificateAdjuster.prototype.extract = function (from, what) {
    var certName = "";
    var begin = from.indexOf(what);
    if (begin >= 0) {
        var end = from.indexOf(', ', begin);
        certName = (end < 0) ? from.substr(begin) : from.substr(begin, end - begin);
    }
    return certName;
};

CertificateAdjuster.prototype.print2Digit = function (digit) {
    return (digit < 10) ? ("0" + digit) : digit;
};

CertificateAdjuster.prototype.getCertDate = function (paramDate) {
    var certDate = new Date(paramDate);
    return this.print2Digit(certDate.getUTCDate()) + "." + this.print2Digit(certDate.getMonth() + 1) + "." + certDate.getFullYear() +
        " " + this.print2Digit(certDate.getUTCHours()) + ":" + this.print2Digit(certDate.getUTCMinutes()) + ":" + this.print2Digit(certDate.getUTCSeconds());
};

CertificateAdjuster.prototype.getCertName = function (certSubjectName) {
    return this.extract(certSubjectName, "CN=");
};

CertificateAdjuster.prototype.getIssuer = function (certIssuerName) {
    return this.extract(certIssuerName, "CN=");
};


Ext.define('Ext.m3.SinopsSigner', {

    extend: 'Ext.util.Observable',

    SIGNATURE_CODES_MAP: {
        'CADES_BES': cadesplugin.CADESCOM_CADES_BES,
        'CADES_X_LONG_TYPE_1': cadesplugin.CADESCOM_CADES_X_LONG_TYPE_1
    },

    PLUGIN_UNAVAILABLE_MSG: (
        "Выполняемое вами действие должно быть подписано с помощью ЭЦП, " +
        "для создания которой необходим установленный \"КриптоПро ЭЦП Browser plug-in\". " +
        "Пожалуйста проверьте настройки браузера!"
    ),

    getCertsList: function (onReady, onFail) {
        var me = this;

        var promise = new Promise(function (resolve, reject) {
            cadesplugin.async_spawn(function *(args) {
                try {
                    var certs = [];

                    var oStore = yield cadesplugin.CreateObjectAsync("CAdESCOM.Store");
                    yield oStore.Open();

                    var certificates = yield oStore.Certificates;
                    var certCnt = yield certificates.Count;
                    var dateObj = new Date();
                    for (var i = 1; i <= certCnt; i++) {
                        var certificate = yield certificates.Item(i);
                        var validToDate = new Date((yield certificate.ValidToDate));
                        var validator = yield certificate.IsValid();
                        var isValid = yield validator.Result;
                        if (dateObj < validToDate && (yield certificate.HasPrivateKey()) && isValid) {
                            var certAdj = new CertificateAdjuster();
                            var pubKey = yield certificate.PublicKey();
                            var algo = yield pubKey.Algorithm;
                            var fAlgoName = yield algo.FriendlyName;
                            var thumbprint = yield certificate.Thumbprint;
                            certs.push({
                                subjectName: certAdj.getCertName(yield certificate.SubjectName),
                                issuerName: certAdj.getIssuer(yield certificate.IssuerName),
                                validFromDate: certAdj.getCertDate(yield certificate.ValidFromDate),
                                validToDate: certAdj.getCertDate(yield certificate.ValidToDate),
                                pubKeyAlgorithm: fAlgoName,
                                thumbprint: thumbprint
                            });
                        }
                    }

                    yield oStore.Close();

                    args[0](certs);
                } catch (ex) {
                    var error = cadesplugin.getLastError(ex);
                    if (error == "Cannot read property 'CreateObjectAsync' of undefined") {
                        error = me.PLUGIN_UNAVAILABLE_MSG;
                    }
                    args[1](error);
                }
            }, resolve, reject);
        });

        promise.then(onReady, onFail);
    },
    // end of getCertsList

    sign: function(thumbprint, content, signOptions, onReady, onFail) {
        var me = this;

        var promise = new Promise(function (resolve, reject) {
            cadesplugin.async_spawn(function *(args) {
                try {
                    var oStore = yield cadesplugin.CreateObjectAsync("CAdESCOM.Store");
                    yield oStore.Open();

                    var CertificatesObj = yield oStore.Certificates;
                    var oCertificates = yield CertificatesObj.Find(cadesplugin.CAPICOM_CERTIFICATE_FIND_SHA1_HASH, thumbprint);
                    var Count = yield oCertificates.Count;
                    var oCertificate;
                    if (Count) {
                        oCertificate = yield oCertificates.Item(1);
                    }

                    var options = Ext.apply({
                        signerOptions: cadesplugin.CAPICOM_CERTIFICATE_INCLUDE_WHOLE_CHAIN,
                        signatureType: cadesplugin.CADESCOM_CADES_BES,
                        detached: false,
                        TSAAddress: me.TSAAddress
                    }, signOptions);

                    var oSigner = yield cadesplugin.CreateObjectAsync("CAdESCOM.CPSigner");
                    yield oSigner.propset_Certificate(oCertificate);
                    yield oSigner.propset_TSAAddress(options.TSAAddress);
                    yield oSigner.propset_Options(options.signerOptions);

                    var oSignedData = yield cadesplugin.CreateObjectAsync("CAdESCOM.CadesSignedData");
                    yield oSignedData.propset_Content(content);

                    me._signedMessage = yield oSignedData.SignCades(oSigner, options.signatureType, options.detached);

                    yield oStore.Close();

                    function invert(obj) {
                        var new_obj = {};
                        for (var prop in obj) {
                            new_obj[obj[prop]] = prop;
                        }
                        return new_obj;
                    }

                    var ctx = {
                        signature_type: invert(me.SIGNATURE_CODES_MAP)[options.signatureType],
                        signature_options: {
                            detached: options.detached,
                            content_encoding: "utf-16-le"
                        }
                    };
                    var signatureParams = {
                        _signed_message: me._signedMessage,
                        _context_data: JSON.stringify(ctx)
                    };

                    args[0](signatureParams);
                }
                catch (ex) {
                    args[1](cadesplugin.getLastError(ex));
                }
            }, resolve, reject);
        });

        promise.then(onReady, onFail);
    },
    // end of sign

    /**
     * Адрес службы штампов времени.
     */
    TSAAddress: "http://testca.cryptopro.ru/tsp/tsp.srf",

    showSuccessMsg: function (msgBoxConfig) {
        var conf = {
            title: "Операция выполнена.",
            msg: "Подпись была успешно проверена. Вы можете " +
                 this.self.generateDownloadLink(this._signedMessage) +
                 " подписанный вами документ.",
            buttons: Ext.MessageBox.OK,
            closable: false,
            icon: Ext.MessageBox.INFO
        };
        conf = Ext.apply(conf, msgBoxConfig);
        return Ext.Msg.show(conf);
    },

    statics: {
        generateDownloadLink: function (base64data, filename, text) {
            filename = filename || "signed_message.der";
            text = text || "скачать документ";
            return (
                '<a download="'+ filename + '"' +
                'href="data:application/octet-stream;charset=utf-8;base64,' +
                base64data + '" >' + text + '</a>'
            );
        }
    }
});
